import type * as _ from '../../../src/Schema/__.js'

export namespace $ {
  export interface Index {
    Root: {
      Query: Root.Query
      Mutation: null
      Subscription: null
    }
    objects: {
      Foo: Object.Foo
      Bar: Object.Bar
      ObjectNested: Object.ObjectNested
      lowerCaseObject: Object.lowerCaseObject
      lowerCaseObject2: Object.lowerCaseObject2
      Object: Object.Object
      Object1ImplementingInterface: Object.Object1ImplementingInterface
      Object2ImplementingInterface: Object.Object2ImplementingInterface
    }
    unions: {
      Union:
        | Union.FooBarUnion
        | Union.lowerCaseUnion
    }
    scalars: Scalars
  }
  export interface Scalars {
    ID: string
    String: string
    Int: number
    Float: number
    Boolean: boolean
  }
}

// ------------------------------------------------------------ //
//                             Root                             //
// ------------------------------------------------------------ //

export namespace Root {
  export type Query = _.Object<'Query', {
    interface: _.Field<_.Output.Nullable<Interface.Interface>>
    id: _.Field<_.Output.Nullable<_.Scalar.ID>>
    idNonNull: _.Field<_.Scalar.ID>
    string: _.Field<_.Output.Nullable<_.Scalar.String>>
    stringWithRequiredArg: _.Field<
      _.Output.Nullable<_.Scalar.String>,
      _.Args<{
        string: _.Scalar.String
      }>
    >
    stringWithArgs: _.Field<
      _.Output.Nullable<_.Scalar.String>,
      _.Args<{
        string: _.Input.Nullable<_.Scalar.String>
        int: _.Input.Nullable<_.Scalar.Int>
        float: _.Input.Nullable<_.Scalar.Float>
        boolean: _.Input.Nullable<_.Scalar.Boolean>
        id: _.Input.Nullable<_.Scalar.ID>
      }>
    >
    stringWithArgEnum: _.Field<
      _.Output.Nullable<_.Scalar.String>,
      _.Args<{
        ABCEnum: _.Input.Nullable<Enum.ABCEnum>
      }>
    >
    stringWithListArg: _.Field<
      _.Output.Nullable<_.Scalar.String>,
      _.Args<{
        ints: _.Input.Nullable<_.Input.List<_.Input.Nullable<_.Scalar.Int>>>
      }>
    >
    stringWithListArgRequired: _.Field<
      _.Output.Nullable<_.Scalar.String>,
      _.Args<{
        ints: _.Input.List<_.Input.Nullable<_.Scalar.Int>>
      }>
    >
    stringWithArgInputObject: _.Field<
      _.Output.Nullable<_.Scalar.String>,
      _.Args<{
        input: _.Input.Nullable<InputObject.InputObject>
      }>
    >
    stringWithArgInputObjectRequired: _.Field<
      _.Output.Nullable<_.Scalar.String>,
      _.Args<{
        input: InputObject.InputObject
      }>
    >
    object: _.Field<_.Output.Nullable<Object.Object>>
    listListIntNonNull: _.Field<_.Output.List<_.Output.List<_.Scalar.Int>>>
    listListInt: _.Field<
      _.Output.Nullable<_.Output.List<_.Output.Nullable<_.Output.List<_.Output.Nullable<_.Scalar.Int>>>>>
    >
    listInt: _.Field<_.Output.Nullable<_.Output.List<_.Output.Nullable<_.Scalar.Int>>>>
    listIntNonNull: _.Field<_.Output.List<_.Scalar.Int>>
    objectNested: _.Field<_.Output.Nullable<Object.ObjectNested>>
    objectNonNull: _.Field<Object.Object>
    objectWithArgs: _.Field<
      _.Output.Nullable<Object.Object>,
      _.Args<{
        string: _.Input.Nullable<_.Scalar.String>
        int: _.Input.Nullable<_.Scalar.Int>
        float: _.Input.Nullable<_.Scalar.Float>
        boolean: _.Input.Nullable<_.Scalar.Boolean>
        id: _.Input.Nullable<_.Scalar.ID>
      }>
    >
    fooBarUnion: _.Field<_.Output.Nullable<Union.FooBarUnion>>
    /**
     * Query enum field documentation.
     */
    abcEnum: _.Field<_.Output.Nullable<Enum.ABCEnum>>
    lowerCaseUnion: _.Field<_.Output.Nullable<Union.lowerCaseUnion>>
  }>
}

// ------------------------------------------------------------ //
//                             Enum                             //
// ------------------------------------------------------------ //

export namespace Enum {
  /**
   * Enum documentation.
   *
   * Members
   * "A" - (DEPRECATED: Enum value A is deprecated.)
   * "B" - Enum B member documentation.
   * "C" - Enum C member documentation. (DEPRECATED: Enum value C is deprecated.)
   */
  export type ABCEnum = _.Enum<'ABCEnum', ['A', 'B', 'C']>
}

// ------------------------------------------------------------ //
//                         InputObject                          //
// ------------------------------------------------------------ //

export namespace InputObject {
  export type InputObject = _.InputObject<'InputObject', {
    id: _.Input.Nullable<_.Scalar.ID>
    idRequired: _.Scalar.ID
  }>
}

// ------------------------------------------------------------ //
//                          Interface                           //
// ------------------------------------------------------------ //

export namespace Interface {
  export type Interface = _.Interface<'Interface', {
    id: _.Field<_.Output.Nullable<_.Scalar.ID>>
  }, [Object.Object1ImplementingInterface, Object.Object2ImplementingInterface]>
}

// ------------------------------------------------------------ //
//                            Object                            //
// ------------------------------------------------------------ //

export namespace Object {
  /**
   * Object documentation.
   */
  export type Foo = _.Object<'Foo', {
    /**
     * Field documentation.
     *
     * @deprecated Field a is deprecated.
     */
    id: _.Field<_.Output.Nullable<_.Scalar.ID>>
  }>

  export type Bar = _.Object<'Bar', {
    int: _.Field<_.Output.Nullable<_.Scalar.Int>>
  }>

  export type ObjectNested = _.Object<'ObjectNested', {
    id: _.Field<_.Output.Nullable<_.Scalar.ID>>
    object: _.Field<_.Output.Nullable<Object.Object>>
  }>

  export type lowerCaseObject = _.Object<'lowerCaseObject', {
    id: _.Field<_.Output.Nullable<_.Scalar.ID>>
  }>

  export type lowerCaseObject2 = _.Object<'lowerCaseObject2', {
    int: _.Field<_.Output.Nullable<_.Scalar.Int>>
  }>

  export type Object = _.Object<'Object', {
    string: _.Field<_.Output.Nullable<_.Scalar.String>>
    int: _.Field<_.Output.Nullable<_.Scalar.Int>>
    float: _.Field<_.Output.Nullable<_.Scalar.Float>>
    boolean: _.Field<_.Output.Nullable<_.Scalar.Boolean>>
    id: _.Field<_.Output.Nullable<_.Scalar.ID>>
  }>

  export type Object1ImplementingInterface = _.Object<'Object1ImplementingInterface', {
    id: _.Field<_.Output.Nullable<_.Scalar.ID>>
    int: _.Field<_.Output.Nullable<_.Scalar.Int>>
  }>

  export type Object2ImplementingInterface = _.Object<'Object2ImplementingInterface', {
    id: _.Field<_.Output.Nullable<_.Scalar.ID>>
    boolean: _.Field<_.Output.Nullable<_.Scalar.Boolean>>
  }>
}

// ------------------------------------------------------------ //
//                            Union                             //
// ------------------------------------------------------------ //

export namespace Union {
  /**
   * Union documentation.
   */
  export type FooBarUnion = _.Union<'FooBarUnion', [Object.Foo, Object.Bar]>

  export type lowerCaseUnion = _.Union<'lowerCaseUnion', [Object.lowerCaseObject, Object.lowerCaseObject2]>
}
